// JNIClient.cpp --- Copyright (c) 1987-2009 by Design Science, Inc.
// purpose: CJNIClient manages JNI environment bookkeeping

#include "StdAfx.h"
#include "JavaVM.h"
#include "JNIClient.h"

CJNIClient::CJNIClient(
) : m_Cls(0), m_Obj(0), m_env(0), m_Initialized(false) {
}

CJNIClient::~CJNIClient(
){
	if (m_env) {
		// retrieve JNI environment and verify cached refs
		RefreshJNIEnv();
		if (m_Cls) m_env->DeleteGlobalRef(m_Cls);
		if (m_Obj) m_env->DeleteGlobalRef(m_Obj);
	}
}

HRESULT 
CJNIClient::InitJavaObject(
	const char * className
){
	Assert(!m_Initialized);

	JavaVM *jvm = theJVM.GetJavaVMPtr();
	if ( jvm== 0) {
		BSTR defaultClasspath(L".");
		BSTR defaultDLLpath(L"jvm");
		theJVM.InitVM(defaultDLLpath, defaultClasspath);
		jvm = theJVM.GetJavaVMPtr();
	}
	Assert(jvm);
	if (jvm==0)
		return E_FAIL;

	jvm->GetEnv((void **)(&m_env), theJVM.version); 

	// load editor class
	jclass localClsRef = m_env->FindClass(className);
	Assert(localClsRef);
	if (localClsRef==0)
		return E_FAIL;

	m_Cls = (jclass)(m_env->NewGlobalRef(localClsRef));

	// lookup constructor
	jmethodID mid = m_env->GetMethodID(m_Cls, "<init>", "()V");
	Assert(mid);
	if (mid==0)
		return E_FAIL;

	// construct object
	jobject localObjectRef = m_env->NewObject(m_Cls, mid); 
	Assert(localObjectRef);
	if (localObjectRef==0)
		return E_FAIL;

	m_Obj = m_env->NewGlobalRef(localObjectRef);

	m_Initialized = true;

	return S_OK;
}

HRESULT 
CJNIClient::RefreshJNIEnv(
){
	Assert(m_Initialized);

	JavaVM *jvm = theJVM.GetJavaVMPtr();
	Assert(jvm);
	if (jvm==0)
		return E_FAIL;

	jvm->GetEnv((void **)(&m_env), theJVM.version); 
	Assert(m_env);
	Assert(m_Cls);
	Assert(m_Obj);
	if (!m_env || !m_Cls || !m_Obj)
		return E_FAIL;

	return S_OK;
}

HRESULT
CJNIClient::CallVoidVoidMethod(
	const char *name
){
	// retrieve JNI environment and verify cached refs
	HRESULT hr = RefreshJNIEnv();
	if (!SUCCEEDED(hr))
		return E_FAIL;

	// lookup the method
	jmethodID mid = m_env->GetMethodID(m_Cls, name, "()V");
	Assert(mid);
	if (mid == 0)
		return E_FAIL;

	// call the method
	m_env->CallVoidMethod(m_Obj, mid); 
	if (m_env->ExceptionOccurred()) 
		return E_FAIL;

	return S_OK;
}

HRESULT
CJNIClient::CallVoidIntMethod(
	const char *name,
	int *retValue
){
	// retrieve JNI environment and verify cached refs
	HRESULT hr = RefreshJNIEnv();
	if (!SUCCEEDED(hr))
		return E_FAIL;

	// lookup the method id
	jmethodID mid = m_env->GetMethodID(m_Cls, name, "()I");
	Assert(mid);
	if (mid == 0)
		return E_FAIL;

	// call the method
	jint val = m_env->CallIntMethod(m_Obj, mid); 
	if (m_env->ExceptionOccurred()) 
		return E_FAIL;

	*retValue = val;

	return S_OK;
}

HRESULT
CJNIClient::CallVoidStringMethod(
	const char *name,
	BSTR *retValue
){
	// retrieve JNI environment and verify cached refs
	HRESULT hr = RefreshJNIEnv();
	if (!SUCCEEDED(hr))
		return E_FAIL;

	// lookup the method id
	jmethodID mid = m_env->GetMethodID(m_Cls, name, "()Ljava/lang/String;");
	Assert(mid);
	if (mid == 0)
		return E_FAIL;

	// call the method
	jobject infoStr = m_env->CallObjectMethod(m_Obj, mid); 
	if (m_env->ExceptionOccurred()) 
		return E_FAIL;

	// convert to a BSTR for return
	const jchar * retChars = m_env->GetStringChars((jstring)infoStr, 0);
	Assert(retChars);
	if (retChars == 0)
		return E_FAIL;
	//directly cast CMJ
	::SysReAllocString(retValue, (OLECHAR *)retChars);

	return S_OK;
}

HRESULT
CJNIClient::CallBoolVoidMethod(
	const char *name,
	BOOL value
){	
	// retrieve JNI environment and verify cached refs
	HRESULT hr = RefreshJNIEnv();
	if (!SUCCEEDED(hr))
		return E_FAIL;

	jmethodID mid = m_env->GetMethodID(m_Cls, name, "(Z)V");
	Assert(mid);
	if (mid == 0)
		return E_FAIL;

	// call the method
	m_env->CallVoidMethod(m_Obj, mid, value); 
	if (m_env->ExceptionOccurred()) 
		return E_FAIL;

	return S_OK;
}

HRESULT
CJNIClient::CallVoidBoolMethod(
	const char *name,
	BOOL* value
){	
	// retrieve JNI environment and verify cached refs
	HRESULT hr = RefreshJNIEnv();
	if (!SUCCEEDED(hr))
		return E_FAIL;

	jmethodID mid = m_env->GetMethodID(m_Cls, name, "()Z");
	Assert(mid);
	if (mid == 0)
		return E_FAIL;

	// call the method
	jboolean val = m_env->CallBooleanMethod(m_Obj, mid); 
	if (m_env->ExceptionOccurred()) 
		return E_FAIL;

	*value = val;
	return S_OK;
}

HRESULT
CJNIClient::CallStringVoidMethod(
	const char *name,
	BSTR value
){	
	// retrieve JNI environment and verify cached refs
	HRESULT hr = RefreshJNIEnv();
	if (!SUCCEEDED(hr))
		return E_FAIL;

	jmethodID mid = m_env->GetMethodID(m_Cls, name, "(Ljava/lang/String;)V");
	Assert(mid);
	if (mid == 0)
		return E_FAIL;

	// marshall the arguments
	USES_CONVERSION;
	jsize len = (jsize) ::SysStringLen(value);
	const jchar *unicodeChars = (jchar *)OLE2CW(value);
	jstring jstr = m_env->NewString(unicodeChars, len);
	Assert(jstr);
	if (jstr == 0)
		return E_FAIL;

	// call the method
	m_env->CallVoidMethod(m_Obj, mid, jstr); 
	if (m_env->ExceptionOccurred()) 
		return E_FAIL;

	return S_OK;
}

HRESULT 
CJNIClient::BSTRToJString(
	BSTR in, 
	jstring *out
){
	// retrieve JNI environment and verify cached refs
	HRESULT hr = RefreshJNIEnv();
	if (!SUCCEEDED(hr))
		return E_FAIL;

	// create the namespace prefix string
	USES_CONVERSION;
	jsize len = (jsize) ::SysStringLen(in);
	const jchar *unicodeChars = (jchar *)OLE2CW(in);
	*out = m_env->NewString(unicodeChars, len);
	Assert(*out);

	return S_OK;
}

HRESULT 
CJNIClient::JStringToBSTR(
	jobject in,
	BSTR *out 
){
	// retrieve JNI environment and verify cached refs
	HRESULT hr = RefreshJNIEnv();
	if (!SUCCEEDED(hr))
		return E_FAIL;

	jstring inStr = (jstring)in;
	const jchar * retChars = m_env->GetStringChars(inStr, 0);
	Assert(retChars);
	if (retChars == 0)
		return E_FAIL;
	::SysReAllocString(out, (OLECHAR *)retChars);
	m_env->ReleaseStringChars(inStr, retChars);

	return S_OK;
}

HWND
CJNIClient::GetHWND(
){
	Assert(m_Initialized);

	JAWT awt;
	JAWT_DrawingSurface* ds;
	JAWT_DrawingSurfaceInfo* dsi;
	JAWT_Win32DrawingSurfaceInfo* dsi_win;
	jint lock;

	HWND returnValue;

	// Get the AWT
	awt.version = JAWT_VERSION_1_3;
	HRESULT hr = theJVM.GetAWT(m_env, &awt);
	Assert(SUCCEEDED(hr));

	// Get the drawing surface
	ds = awt.GetDrawingSurface(m_env, m_Obj);
	if(ds == NULL) {
		// bail out before committing any resources
		return NULL; 
	}

	// Lock the drawing surface
	lock = ds->Lock(ds);
	Assert((lock & JAWT_LOCK_ERROR) == 0);

	// Get the drawing surface info
	dsi = ds->GetDrawingSurfaceInfo(ds);

	// Get the platform-specific drawing info
	dsi_win = (JAWT_Win32DrawingSurfaceInfo*)dsi->platformInfo;

	if(dsi_win == NULL)
		returnValue = NULL;
	else 
		returnValue = dsi_win->hwnd;
 
	// Free the drawing surface info
	ds->FreeDrawingSurfaceInfo(dsi);
	// Unlock the drawing surface
	ds->Unlock(ds);
	// Free the drawing surface
	awt.FreeDrawingSurface(ds);

	return returnValue;
}
